@@ -209,6 +209,50 @@ uint8_t* generate_nds_file(size_t mb, unsigned arm9_size, unsigned arm7_size, si
   return image;
 }
 
+uint8_t* generate_gamecube_iso(size_t mb, size_t* image_size)
+{
+  uint8_t* image;
+  const size_t size_needed = mb * 1024 * 1024;
+  int ix;
+
+  image = (uint8_t*)calloc(size_needed, 1);
+  if (image != NULL)
+  {
+    uint32_t apploader_sizes_addr = 0x2440 + 0x14;
+    uint32_t dol_offset_addr = 0x420;
+    uint32_t dol_sizes_addr = 0x3000;
+
+    fill_image(image, size_needed);
+
+    image[0x1c] = 0xC2;
+    image[0x1d] = 0x33;
+    image[0x1e] = 0x9F;
+    image[0x1f] = 0x3D;
+
+    for (ix = 0; ix < 8; ix++)
+    {
+      /* 0x000000ff for both */
+      image[apploader_sizes_addr + ix] = (ix % 4 == 3) ? 0xff : 0;
+    }
+    for (ix = 0; ix < 4; ix++)
+    {
+      /* 0x00003000 */
+      image[dol_offset_addr + ix] = (ix % 4 == 2) ? 0x30 : 0; 
+    }
+    for (ix = 0; ix < 18 * 4; ix++)
+    {
+      /* offsets start at 0x00003100 and increment */
+      image[dol_sizes_addr + ix] = (ix % 4 == 2) ? (0x30 + 1 + ix / 4) : 0; 
+      /* 0x000000ff for every other size */
+      image[dol_sizes_addr + 0x90 + ix] = (ix % 8 == 3) ? 0xff : 0; 
+    }
+  }
+
+  if (image_size)
+    *image_size = size_needed;
+  return image;
+}
+
 uint8_t* generate_atari_7800_file(size_t kb, int with_header, size_t* image_size)
 {
   uint8_t* image;
@@ -310,6 +354,9 @@ uint8_t* generate_3do_bin(unsigned root_directory_sectors, unsigned binary_size,
   size_t offset = 2048;
   unsigned i;
 
+  if (!image)
+    return NULL;
+
   /* first sector - volume header */
   memcpy(image, volume_header, sizeof(volume_header));
   image[0x5B] = (uint8_t)root_directory_sectors;
@@ -393,6 +440,8 @@ uint8_t* generate_dreamcast_bin(unsigned track_first_sector, unsigned binary_siz
   const size_t binary_sectors = (binary_size + 2047) / 2048;
   const size_t size_needed = (binary_sectors + 18) * 2048;
   uint8_t* image = (uint8_t*)calloc(size_needed, 1);
+  if (!image)
+    return NULL;
 
   /* volume header goes in sector 0 */
   memcpy(&image[0], volume_header, sizeof(volume_header));
@@ -442,6 +491,8 @@ uint8_t* generate_pce_cd_bin(unsigned binary_sectors, size_t* image_size)
 
   size_t size_needed = (binary_sectors + 2) * 2048;
   uint8_t* image = (uint8_t*)calloc(size_needed, 1);
+  if (!image)
+    return NULL;
 
   /* volume header goes in second sector */
   memcpy(&image[2048], volume_header, sizeof(volume_header));
@@ -476,9 +527,11 @@ uint8_t* generate_pcfx_bin(unsigned binary_sectors, size_t* image_size)
 
   size_t size_needed = (binary_sectors + 2) * 2048;
   uint8_t* image = (uint8_t*)calloc(size_needed, 1);
+  if (!image)
+    return NULL;
 
   /* volume header goes in second sector */
-  strcpy((char*)&image[0], "PC-FX:Hu_CD-ROM");
+  strcpy_s((char*)&image[0], size_needed, "PC-FX:Hu_CD-ROM");
   memcpy(&image[2048], volume_header, sizeof(volume_header));
   image[2048 + 0x24] = (uint8_t)binary_sectors;
 
@@ -497,6 +550,9 @@ uint8_t* generate_iso9660_bin(unsigned num_sectors, const char* volume_label, si
   
   *image_size = num_sectors * 2048;
   image = calloc(*image_size, 1);
+  if (!image)
+    return NULL;
+
   volume_descriptor = &image[16 * 2048];
 
   /* CD001 identifier */
@@ -638,6 +694,9 @@ uint8_t* generate_jaguarcd_bin(unsigned header_offset, unsigned binary_size, int
   uint8_t* image = (uint8_t*)calloc(size_needed, 1);
   int i;
 
+  if (!image)
+    return NULL;
+
   /* header is 64 bytes of ATRI repeating followed by approved data message, load address, and binary size */
   for (i = 0; i < 64; i += 4)
       memcpy(&image[header_offset + i], "ATRI", 4);
@@ -720,7 +779,7 @@ uint8_t* generate_generic_file(size_t size)
 
 uint8_t* convert_to_2352(uint8_t* input, size_t* size, uint32_t first_sector)
 {
-    const uint32_t num_sectors = (uint32_t)(*size / 2048);
+    const uint32_t num_sectors = (uint32_t)((*size + 2047) / 2048);
     const uint32_t output_size = num_sectors * 2352;
     const uint8_t sync_pattern[] = {
       0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00
