@@ -1,5 +1,6 @@
 #include "rc_hash.h"
 
+#include "../rcheevos/rc_compat.h"
 #include "../test_framework.h"
 #include "data.h"
 #include "mock_filereader.h"
@@ -672,6 +673,71 @@ static void test_hash_atari_jaguar_cd_no_sessions()
   ASSERT_NUM_EQUALS(result_iterator, 0);
 }
 
+extern const char* _rc_hash_jaguar_cd_homebrew_hash;
+
+static void test_hash_atari_jaguar_cd_homebrew()
+{
+  /* Jaguar CD homebrew games all appear to have a common bootloader in the primary boot executable space. They only
+   * differ in a secondary executable in the second track (part of the first session). This doesn't appear to be
+   * intentional behavior based on the CD BIOS documentation, which states that all developer code should be in the
+   * first track of the second session. I speculate this is done to work around the authentication logic. */
+  const char* cue_file =
+      "REM SESSION 01\n"
+      "FILE \"track01.bin\" BINARY\n"
+      "  TRACK 01 AUDIO\n"
+      "    INDEX 01 00:00:00\n"
+      "FILE \"track02.bin\" BINARY\n"
+      "  TRACK 02 AUDIO\n"
+      "    INDEX 01 00:00:00\n"
+      "REM SESSION 02\n"
+      "FILE \"track03.bin\" BINARY\n"
+      "  TRACK 03 AUDIO\n"
+      "    INDEX 01 00:00:00\n";
+  size_t image_size, image_size2;
+  uint8_t* image = generate_jaguarcd_bin(2, 45760, 1, &image_size);
+  uint8_t* image2 = generate_jaguarcd_bin(2, 986742, 1, &image_size2);
+  char hash_file[33], hash_iterator[33];
+  const char* expected_md5 = "3fdf70e362c845524c9e447aacaed0a9";
+
+  image2[0x60] = 0x21; /* ATARI APPROVED DATA HEADER ATRI! */
+  memcpy(&image2[0xA2], &image2[0x62], 8); /* addr / size */
+  memcpy(&image2[0x62], "RTKARTKARTKARTKA", 16); /* KARTKARTKARTKART */
+  memcpy(&image2[0x72], "RTKARTKARTKARTKA", 16);
+  memcpy(&image2[0x82], "RTKARTKARTKARTKA", 16);
+  memcpy(&image2[0x92], "RTKARTKARTKARTKA", 16);
+
+  mock_file(0, "game.cue", (uint8_t*)cue_file, strlen(cue_file));
+  mock_file(2, "track02.bin", image2, image_size2);
+  mock_file(1, "track03.bin", image, image_size);
+
+  rc_hash_init_default_cdreader(); /* want to test actual FIRST_OF_SECOND_SESSION calculation */
+  _rc_hash_jaguar_cd_homebrew_hash = "4e4114b2675eff21bb77dd41e141ddd6"; /* mock the hash of the homebrew bootloader */
+
+  /* test file hash */
+  int result_file = rc_hash_generate_from_file(hash_file, RC_CONSOLE_ATARI_JAGUAR_CD, "game.cue");
+
+  /* test file identification from iterator */
+  int result_iterator;
+  struct rc_hash_iterator iterator;
+
+  rc_hash_initialize_iterator(&iterator, "game.cue", NULL, 0);
+  result_iterator = rc_hash_iterate(hash_iterator, &iterator);
+  rc_hash_destroy_iterator(&iterator);
+
+  /* cleanup */
+  _rc_hash_jaguar_cd_homebrew_hash = NULL;
+  free(image);
+  free(image2);
+  init_mock_cdreader();
+
+  /* validation */
+  ASSERT_NUM_EQUALS(result_file, 1);
+  ASSERT_STR_EQUALS(hash_file, expected_md5);
+
+  ASSERT_NUM_EQUALS(result_iterator, 1);
+  ASSERT_STR_EQUALS(hash_iterator, expected_md5);
+}
+
 /* ========================================================================= */
 
 static void test_hash_dreamcast_single_bin()
@@ -804,6 +870,22 @@ static void test_hash_dreamcast_cue()
 
 /* ========================================================================= */
 
+static void test_hash_gamecube()
+{
+  size_t image_size;
+  uint8_t* image = generate_gamecube_iso(32, &image_size);
+  mock_file(0, "test.iso", image, image_size);
+  char hash[33];
+  int result = rc_hash_generate_from_file(hash, RC_CONSOLE_GAMECUBE, "test.iso");
+  free(image);
+
+  ASSERT_NUM_EQUALS(result, 1);
+  ASSERT_STR_EQUALS(hash, "c7803b704fa43d22d8f6e55f4789cb45");
+  ASSERT_NUM_EQUALS(image_size, 32 * 1024 * 1024);
+}
+
+/* ========================================================================= */
+
 static void test_hash_nes_32k()
 {
   size_t image_size;
@@ -998,13 +1080,17 @@ static void test_hash_nds()
 
 static void test_hash_nds_supercard()
 {
-  size_t image_size;
+  size_t image_size, image2_size;
   uint8_t* image = generate_nds_file(2, 1234567, 654321, &image_size);
   char hash_file[33], hash_iterator[33];
   const char* expected_hash = "56b30c276cba4affa886bd38e8e34d7e";
+  ASSERT_PTR_NOT_NULL(image);
+  ASSERT_NUM_GREATER(image_size, 0);
 
   /* inject the SuperCard header (512 bytes) */
-  uint8_t* image2 = malloc(image_size + 512);
+  image2_size = image_size + 512;
+  uint8_t* image2 = malloc(image2_size);
+  ASSERT_PTR_NOT_NULL(image2);
   memcpy(&image2[512], &image[0], image_size);
   memset(&image2[0], 0, 512);
   image2[0] = 0x2E;
@@ -1016,7 +1102,7 @@ static void test_hash_nds_supercard()
   image2[0xB2] = 0x96;
   image2[0xB3] = 0x00;
 
-  mock_file(0, "game.nds", image2, image_size);
+  mock_file(0, "game.nds", image2, image2_size);
 
   /* test file hash */
   int result_file = rc_hash_generate_from_file(hash_file, RC_CONSOLE_NINTENDO_DS, "game.nds");
@@ -1070,6 +1156,67 @@ static void test_hash_nds_buffered()
   ASSERT_STR_EQUALS(hash_iterator, expected_hash);
 }
 
+/* ========================================================================= */
+
+static void test_hash_dsi()
+{
+  size_t image_size;
+  uint8_t* image = generate_nds_file(2, 1234567, 654321, &image_size);
+  char hash_file[33], hash_iterator[33];
+  const char* expected_hash = "56b30c276cba4affa886bd38e8e34d7e";
+
+  mock_file(0, "game.nds", image, image_size);
+
+  /* test file hash */
+  int result_file = rc_hash_generate_from_file(hash_file, RC_CONSOLE_NINTENDO_DSI, "game.nds");
+
+  /* test file identification from iterator */
+  int result_iterator;
+  struct rc_hash_iterator iterator;
+
+  rc_hash_initialize_iterator(&iterator, "game.nds", NULL, 0);
+  result_iterator = rc_hash_iterate(hash_iterator, &iterator);
+  rc_hash_destroy_iterator(&iterator);
+
+  /* cleanup */
+  free(image);
+
+  /* validation */
+  ASSERT_NUM_EQUALS(result_file, 1);
+  ASSERT_STR_EQUALS(hash_file, expected_hash);
+
+  ASSERT_NUM_EQUALS(result_iterator, 1);
+  ASSERT_STR_EQUALS(hash_iterator, expected_hash);
+}
+
+static void test_hash_dsi_buffered()
+{
+  size_t image_size;
+  uint8_t* image = generate_nds_file(2, 1234567, 654321, &image_size);
+  char hash_buffer[33], hash_iterator[33];
+  const char* expected_hash = "56b30c276cba4affa886bd38e8e34d7e";
+
+  /* test file hash */
+  int result_buffer = rc_hash_generate_from_buffer(hash_buffer, RC_CONSOLE_NINTENDO_DSI, image, image_size);
+
+  /* test file identification from iterator */
+  int result_iterator;
+  struct rc_hash_iterator iterator;
+
+  rc_hash_initialize_iterator(&iterator, "game.nds", image, image_size);
+  result_iterator = rc_hash_iterate(hash_iterator, &iterator);
+  rc_hash_destroy_iterator(&iterator);
+
+  /* cleanup */
+  free(image);
+
+  /* validation */
+  ASSERT_NUM_EQUALS(result_buffer, 1);
+  ASSERT_STR_EQUALS(hash_buffer, expected_hash);
+
+  ASSERT_NUM_EQUALS(result_iterator, 1);
+  ASSERT_STR_EQUALS(hash_iterator, expected_hash);
+}
 
 /* ========================================================================= */
 
@@ -1155,6 +1302,45 @@ static void test_hash_neogeocd_multiple_prg()
   ASSERT_STR_EQUALS(hash_iterator, expected_md5);
 }
 
+static void test_hash_neogeocd_lowercase_ipl_contents()
+{
+  const char* ipl_txt = "fixa.fix,0,0\r\nprog.prg,0,0\r\nsound.pcm,0,0\r\n\x1a";
+  const size_t prog_prg_size = 273470;
+  uint8_t* prog_prg = generate_generic_file(prog_prg_size);
+  size_t image_size;
+  uint8_t* image = generate_iso9660_bin(160, "TEST", &image_size);
+  char hash_file[33], hash_iterator[33];
+  const char* expected_md5 = "96f35b20c6cf902286da45e81a50b2a3";
+
+  generate_iso9660_file(image, "IPL.TXT", (uint8_t*)ipl_txt, strlen(ipl_txt));
+  generate_iso9660_file(image, "PROG.PRG", prog_prg, prog_prg_size);
+
+  mock_file(0, "game.bin", image, image_size);
+  mock_file(1, "game.cue", (uint8_t*)"game.bin", 8);
+
+  /* test file hash */
+  int result_file = rc_hash_generate_from_file(hash_file, RC_CONSOLE_NEO_GEO_CD, "game.cue");
+
+  /* test file identification from iterator */
+  int result_iterator;
+  struct rc_hash_iterator iterator;
+
+  rc_hash_initialize_iterator(&iterator, "game.cue", NULL, 0);
+  result_iterator = rc_hash_iterate(hash_iterator, &iterator);
+  rc_hash_destroy_iterator(&iterator);
+
+  /* cleanup */
+  free(image);
+  free(prog_prg);
+
+  /* validation */
+  ASSERT_NUM_EQUALS(result_file, 1);
+  ASSERT_STR_EQUALS(hash_file, expected_md5);
+
+  ASSERT_NUM_EQUALS(result_iterator, 1);
+  ASSERT_STR_EQUALS(hash_iterator, expected_md5);
+}
+
 /* ========================================================================= */
 
 static void test_hash_pce_cd()
@@ -1789,8 +1975,8 @@ static void test_hash_m3u_relative_path()
 
 static void test_hash_m3u_absolute_path(const char* absolute_path)
 {
-  char m3u_contents[128] = "#EXTM3U\r\n\r\n#EXTBYT:131072\r\n";
-  strcat(m3u_contents, absolute_path);
+  char m3u_contents[128];
+  snprintf(m3u_contents, sizeof(m3u_contents), "#EXTM3U\r\n\r\n#EXTBYT:131072\r\n%s", absolute_path);
 
   assert_valid_m3u(absolute_path, "relative/test.m3u", m3u_contents);
 }
@@ -1910,6 +2096,7 @@ void test_hash(void) {
   TEST(test_hash_atari_jaguar_cd_track3);
   TEST(test_hash_atari_jaguar_cd_no_header);
   TEST(test_hash_atari_jaguar_cd_no_sessions);
+  TEST(test_hash_atari_jaguar_cd_homebrew);
 
   /* Colecovision */
   TEST_PARAMS4(test_hash_full_file, RC_CONSOLE_COLECOVISION, "test.col", 16384, "455f07d8500f3fabc54906737866167f");
@@ -1941,6 +2128,9 @@ void test_hash(void) {
   /* Gameboy Advance */
   TEST_PARAMS4(test_hash_full_file, RC_CONSOLE_GAMEBOY_COLOR, "test.gba", 4194304, "a247ec8a8c42e18fcb80702dfadac14b");
 
+  /* Gamecube */
+  TEST(test_hash_gamecube);
+
   /* Game Gear */
   TEST_PARAMS4(test_hash_full_file, RC_CONSOLE_GAME_GEAR, "test.gg", 524288, "68f0f13b598e0b66461bc578375c3888");
 
@@ -1970,6 +2160,7 @@ void test_hash(void) {
   /* Neo Geo CD */
   TEST(test_hash_neogeocd);
   TEST(test_hash_neogeocd_multiple_prg);
+  TEST(test_hash_neogeocd_lowercase_ipl_contents);
 
   /* Neo Geo Pocket */
   TEST_PARAMS4(test_hash_full_file, RC_CONSOLE_NEOGEO_POCKET, "test.ngc", 2097152, "cf86acf519625a25a17b1246975e90ae");
@@ -2001,6 +2192,10 @@ void test_hash(void) {
   TEST(test_hash_nds_supercard);
   TEST(test_hash_nds_buffered);
 
+  /* Nintendo DSi */
+  TEST(test_hash_dsi);
+  TEST(test_hash_dsi_buffered);
+
   /* Oric (no fixed file size) */
   TEST_PARAMS4(test_hash_full_file, RC_CONSOLE_ORIC, "test.tap", 18119, "953a2baa3232c63286aeae36b2172cef");
 
@@ -2056,9 +2251,16 @@ void test_hash(void) {
   TEST_PARAMS4(test_hash_full_file, RC_CONSOLE_SUPER_NINTENDO, "test.smc", 524288, "68f0f13b598e0b66461bc578375c3888");
   TEST_PARAMS4(test_hash_full_file, RC_CONSOLE_SUPER_NINTENDO, "test.smc", 524288 + 512, "258c93ebaca1c3f488ab48218e5e8d38");
 
+  /* TI-83 */
+  TEST_PARAMS4(test_hash_full_file, RC_CONSOLE_TI83, "test.83g", 1695, "bfb6048395a425c69743900785987c42");
+  TEST_PARAMS4(test_hash_full_file, RC_CONSOLE_TI83, "test.83p", 2500, "6e81d530ee9a79d4f4f505729ad74bb5");
+
   /* TIC-80 */
   TEST_PARAMS4(test_hash_full_file, RC_CONSOLE_TIC80, "test.tic", 67682, "79b96f4ffcedb3ce8210a83b22cd2c69");
 
+  /* Uzebox */
+  TEST_PARAMS4(test_hash_full_file, RC_CONSOLE_UZEBOX, "test.uze", 53654, "a9aab505e92edc034d3c732869159789");
+
   /* Vectrex */
   TEST_PARAMS4(test_hash_full_file, RC_CONSOLE_SG1000, "test.vec", 4096, "572686c3a073162e4ec6eff86e6f6e3a");
 
